@@ -867,7 +867,13 @@ _pygi_argument_from_object (PyObject   *object,
     switch (type_tag) {
         case GI_TYPE_TAG_VOID:
             g_warn_if_fail (transfer == GI_TRANSFER_NOTHING);
-            arg.v_pointer = object;
+            if (object == Py_None) {
+                Py_DECREF(Py_None);
+                arg.v_pointer = NULL;
+            } else {
+                /* This will leak a reference to the python object. */
+                arg.v_pointer = object;
+            }
             break;
         case GI_TYPE_TAG_BOOLEAN:
         {
@@ -1652,7 +1658,7 @@ _pygi_argument_to_object (GIArgument  *arg,
             GITypeTag item_type_tag;
             GITransfer item_transfer;
             gsize i, item_size;
-            
+
             if (arg->v_pointer == NULL)
                 return PyList_New (0);
             
@@ -1673,41 +1679,30 @@ _pygi_argument_to_object (GIArgument  *arg,
 
             if (item_type_tag == GI_TYPE_TAG_UINT8) {
                 /* Return as a byte array */
-                if (arg->v_pointer == NULL) {
-                    object = PYGLIB_PyBytes_FromString ("");
-                    break;
-                }
-
-                object = PYGLIB_PyBytes_FromStringAndSize(array->data, array->len);
-                break;
-
+                object = PYGLIB_PyBytes_FromStringAndSize (array->data, array->len);
             } else {
-                if (arg->v_pointer == NULL) {
-                    object = PyList_New (0);
-                    break;
-                }
-
                 object = PyList_New (array->len);
                 if (object == NULL) {
+                    g_critical ("Failure to allocate array for %u items", array->len);
+                    g_base_info_unref ( (GIBaseInfo *) item_type_info);
                     break;
                 }
 
-            }            
+                for (i = 0; i < array->len; i++) {
+                    GIArgument item = { 0 };
+                    PyObject *py_item;
+                    
+                    memcpy (&item, array->data + i * item_size, item_size);
 
-            for (i = 0; i < array->len; i++) {
-                GIArgument item = { 0 };
-                PyObject *py_item;
-                
-                memcpy (&item, array->data + i * item_size, item_size);
+                    py_item = _pygi_argument_to_object (&item, item_type_info, item_transfer);
+                    if (py_item == NULL) {
+                        Py_CLEAR (object);
+                        _PyGI_ERROR_PREFIX ("Item %zu: ", i);
+                        break;
+                    }
 
-                py_item = _pygi_argument_to_object (&item, item_type_info, item_transfer);
-                if (py_item == NULL) {
-                    Py_CLEAR (object);
-                    _PyGI_ERROR_PREFIX ("Item %zu: ", i);
-                    break;
+                    PyList_SET_ITEM (object, i, py_item);
                 }
-
-                PyList_SET_ITEM (object, i, py_item);
             }
 
             g_base_info_unref ( (GIBaseInfo *) item_type_info);
@@ -1838,6 +1833,11 @@ _pygi_argument_to_object (GIArgument  *arg,
                         break;
                     }
 
+                    if (G_IS_PARAM_SPEC (arg->v_pointer)) {
+                      object = pyg_param_spec_new (arg->v_pointer);
+                      break;
+                    }
+
                     /* since we will unref the object when the
                      * wrapper is destroyed and we don't want
                      * GTK removing the object while the
@@ -2056,7 +2056,10 @@ _pygi_argument_from_g_value(const GValue *value,
                     break;
                 case GI_INFO_TYPE_INTERFACE:
                 case GI_INFO_TYPE_OBJECT:
-                    arg.v_pointer = g_value_get_object (value);
+                    if (G_VALUE_HOLDS_PARAM (value))
+                      arg.v_pointer = g_value_get_param (value);
+                    else
+                      arg.v_pointer = g_value_get_object (value);
                     break;
                 case GI_INFO_TYPE_BOXED:
                 case GI_INFO_TYPE_STRUCT:
