@@ -3,9 +3,9 @@
 # vim: tabstop=4 shiftwidth=4 expandtab
 
 import unittest
+import traceback
 
 import sys
-sys.path.insert(0, "../")
 from sys import getrefcount
 
 import copy
@@ -229,6 +229,20 @@ class TestEverything(unittest.TestCase):
         self.assertEqual(getrefcount(l1), init_refcount + 1)
         self.assertEqual(getrefcount(l3), init_refcount)
 
+    def test_struct_opaque(self):
+        # we should get a sensible error message
+        try:
+            Everything.TestBoxedPrivate()
+            self.fail('allocating disguised struct without default constructor unexpectedly succeeded')
+        except TypeError:
+            (e_type, e_value, e_tb) = sys.exc_info()
+            self.assertEqual(e_type, TypeError)
+            self.assertTrue('TestBoxedPrivate' in str(e_value), str(e_value))
+            self.assertTrue('override' in str(e_value), str(e_value))
+            self.assertTrue('constructor' in str(e_value), str(e_value))
+            tb = ''.join(traceback.format_exception(e_type, e_value, e_tb))
+            self.assertTrue('tests/test_everything.py", line' in tb, tb)
+
 
 class TestNullableArgs(unittest.TestCase):
     def test_in_nullable_hash(self):
